"use strict";
const path = require("path");
const semver = require("semver");
const constants = require("./constants");
const helpers_1 = require("../lib/common/helpers");
class NpmInstallationManager {
    constructor($npm, $logger, $lockfile, $errors, $options, $fs, $staticConfig) {
        this.$npm = $npm;
        this.$logger = $logger;
        this.$lockfile = $lockfile;
        this.$errors = $errors;
        this.$options = $options;
        this.$fs = $fs;
        this.$staticConfig = $staticConfig;
    }
    getLatestVersion(packageName) {
        return (() => {
            return this.getVersion(packageName, constants.PackageVersion.LATEST).wait();
        }).future()();
    }
    getNextVersion(packageName) {
        return (() => {
            return this.getVersion(packageName, constants.PackageVersion.NEXT).wait();
        }).future()();
    }
    getLatestCompatibleVersion(packageName) {
        return (() => {
            let cliVersionRange = `~${this.$staticConfig.version}`;
            let latestVersion = this.getLatestVersion(packageName).wait();
            if (semver.satisfies(latestVersion, cliVersionRange)) {
                return latestVersion;
            }
            let data = this.$npm.view(packageName, { json: true, "versions": true }).wait();
            return semver.maxSatisfying(data, cliVersionRange) || latestVersion;
        }).future()();
    }
    install(packageName, projectDir, opts) {
        return (() => {
            while (this.$lockfile.check().wait()) {
                helpers_1.sleep(10);
            }
            this.$lockfile.lock().wait();
            try {
                let packageToInstall = this.$options.frameworkPath || packageName;
                let pathToSave = projectDir;
                let version = (opts && opts.version) || null;
                let dependencyType = (opts && opts.dependencyType) || null;
                return this.installCore(packageToInstall, pathToSave, version, dependencyType).wait();
            }
            catch (error) {
                this.$logger.debug(error);
                this.$errors.fail("%s. Error: %s", NpmInstallationManager.NPM_LOAD_FAILED, error);
            }
            finally {
                this.$lockfile.unlock().wait();
            }
        }).future()();
    }
    installCore(packageName, pathToSave, version, dependencyType) {
        return (() => {
            const possiblePackageName = path.resolve(packageName);
            if (this.$fs.exists(possiblePackageName).wait()) {
                packageName = possiblePackageName;
            }
            if (packageName.indexOf(".tgz") >= 0) {
                version = null;
            }
            if (this.isURL(packageName) || this.$fs.exists(packageName).wait()) {
                version = null;
            }
            else {
                version = version || this.getLatestCompatibleVersion(packageName).wait();
            }
            let installedModuleNames = this.npmInstall(packageName, pathToSave, version, dependencyType).wait();
            let installedPackageName = installedModuleNames[0];
            let pathToInstalledPackage = path.join(pathToSave, "node_modules", installedPackageName);
            return pathToInstalledPackage;
        }).future()();
    }
    isURL(str) {
        let urlRegex = '^(?!mailto:)(?:(?:http|https|ftp)://)(?:\\S+(?::\\S*)?@)?(?:(?:(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}(?:\\.(?:[0-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))|(?:(?:[a-z\\u00a1-\\uffff0-9]+-?)*[a-z\\u00a1-\\uffff0-9]+)(?:\\.(?:[a-z\\u00a1-\\uffff0-9]+-?)*[a-z\\u00a1-\\uffff0-9]+)*(?:\\.(?:[a-z\\u00a1-\\uffff]{2,})))|localhost)(?::\\d{2,5})?(?:(/|\\?|#)[^\\s]*)?$';
        let url = new RegExp(urlRegex, 'i');
        return str.length < 2083 && url.test(str);
    }
    npmInstall(packageName, pathToSave, version, dependencyType) {
        return (() => {
            this.$logger.out("Installing ", packageName);
            packageName = packageName + (version ? `@${version}` : "");
            let npmOptions = { silent: true };
            if (dependencyType) {
                npmOptions[dependencyType] = true;
            }
            return this.$npm.install(packageName, pathToSave, npmOptions).wait();
        }).future()();
    }
    getVersion(packageName, version) {
        return (() => {
            let data = this.$npm.view(packageName, { json: true, "dist-tags": true }).wait();
            this.$logger.trace("Using version %s. ", data[version]);
            return data[version];
        }).future()();
    }
}
NpmInstallationManager.NPM_LOAD_FAILED = "Failed to retrieve data from npm. Please try again a little bit later.";
exports.NpmInstallationManager = NpmInstallationManager;
$injector.register("npmInstallationManager", NpmInstallationManager);
